#!/usr/bin/python

import os
import glob
import configparser


# find all languages in "base" directory
def find_langs(base_dir):
    langs = []
    path = os.path.join(base_dir, 'base', 'locale')
    for root, dirs, filenames in os.walk(path):
        for dir in dirs:
            langs.append(dir)
            print(dir)
        break

    return langs

# get a list of all *.cfg filenames for the given language
def find_lang_paths(base_dir, lang):
    paths = []
    # walk over the module directories
    for root, dirs, filenames in os.walk(base_dir):
        for dir in dirs:
            # read all cfg files
            found_files = glob.glob(os.path.join(base_dir, dir, 'locale', lang, '*.cfg'))
            paths += found_files
        break
    return paths

# read given files as ini files
# return a dictionary that contains the merged data from all files
def read_cfg(paths):
    data = {}
    config = configparser.ConfigParser(allow_unnamed_section=True, interpolation=None)
    config.read(paths, encoding='utf-8')
    for section in config.sections():
        data[section] = dict(config.items(section))

    return data

def create_reference_data(data, section_names):
    return { key: data['en'][key] for key in section_names if key in data['en'] }

# build the "| <english term> = <language term>" template lines
def create_template(data, lang, section_names, cfg):

    tpldata = []
    keys = {}
    for section_name in section_names:
        section_en = data['en'][section_name]
        trailing_en = ""

        trailing = ""

        if section_name in data[lang]:
            section = data[lang][section_name]
            if section_name == 'technology-name':
                trailing_en = f' ({cfg['en']['research']})'
                trailing = f' ({lang in cfg and 'research' in cfg[lang] and cfg[lang]['research'] or cfg['en']['research']})'

            for key, value in section.items():
                value_en = section_en[key]
                if '__' in value_en:
                    continue
                if section_name in cfg and key in cfg[section_name]:
                    continue
                if value_en in keys:
                    continue
                keys[value_en] = 1
                tpldata.append(f'| {value_en}{trailing_en} = {value}{trailing}')

    return tpldata

# as reference, with no real value for the wiki, create sections with <english term> = <language term> to see how the language strings are correlated
def create_ini(data, lang, section_names, cfg):

    inidata = {}
    for section_name in section_names:
        section_en = data['en'][section_name]

        inidata[section_name] = {}

        if section_name in data[lang]:
            section = data[lang][section_name]

            for key, value in section.items():
                value_en = section_en[key]
                if '__' in value_en:
                    continue
                if section_name in cfg and key in cfg[section_name]:
                    continue
                inidata[section_name][value_en] = value

    return inidata

def save_tpl(output_dir, data, filename):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    with open(os.path.join(output_dir, filename), 'w', encoding='utf-8') as file:
        for line in sorted(data):
            file.write(f'{line}\n')

def save_ini(output_dir, data, filename):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    with open(os.path.join(output_dir, filename), 'w', encoding='utf-8') as file:
        for section, items in data.items():
            file.write(f'[{section}]\n')
            for key, value in items.items():
                file.write(f'{key} = {value}\n')
            file.write('\n')


def main():
    base_dir = r'C:\Program Files (x86)\Steam\steamapps\common\Factorio\data'
    output_dir = r'd:\temp\t\ProcessedFiles'
 
    sections_for_template = [ 'technology-name' ,'recipe-name', 'entity-name', 'equipment-name', 'fluid-name', 'item-name' ]

    data = {}
    inidata = {}
    tpldata = {}

    cfg = read_cfg(r'.\create-factorio-language-templates.cfg')

    langs = find_langs(base_dir)

    for lang in langs:
        print('read language ' + lang)
        paths = find_lang_paths(base_dir, lang)
        data[lang] = read_cfg(paths)

    # process data
    # this must be separate from the first iteration, because we need language en for keywords
    for lang in langs:
        inidata[lang] = create_ini(data, lang, sections_for_template, cfg)
        tpldata[lang] = create_template(data, lang, sections_for_template, cfg)

    # write processed data to disk
    for lang in langs:
        save_tpl(output_dir, tpldata[lang], lang + '-tpl.cfg')
        save_ini(output_dir, inidata[lang], lang + '-ini.cfg')

    # write a filtered ini-formatted file with keyword = english text sections
    reference = create_reference_data(data, sections_for_template)
    save_ini(output_dir, reference, '0reference.cfg')

if __name__ == '__main__':
    main()
